@@ -34,7 +34,7 @@ from calculate.lib.utils.files import (isMount, removeDir,
                                        listDirectory, checkUtils,
                                        MAGIC_COMPRESS, MAGIC_SYMLINK,
                                        MAGIC_CONTINUE, makeDirectory,
-                                       isEmpty,
+                                       isEmpty, check_rw,
                                        PercentProgress, getProgPath)
 from calculate.lib.utils.device import (detectDeviceForPartition,
                                         getUdevDeviceInfo, countPartitions)
@@ -43,6 +43,7 @@ from calculate.lib.utils.text import _u8
 from calculate.lib.variables.linux import LinuxDataVars, Linux
 
 from calculate.lib.cl_lang import setLocalTranslate, _
+from ldap import dn
 
 setLocalTranslate('cl_install3', sys.modules[__name__])
 
@@ -79,7 +80,7 @@ class cpProcessProgress(processProgress):
     def init(self, *args, **kwargs):
         self.maxfiles = kwargs.get('maxfiles', 1)
         self.stderr = STDOUT
-        self.command.append("-av")
+        self.command.append("-dRv")
 
     def processInit(self):
         self.value = 0
@@ -92,8 +93,8 @@ class cpProcessProgress(processProgress):
         return 100
 
     def processString(self, strdata):
-        if strdata.startswith("cp:"):
-            self.cacheresult.append(strdata[4:])
+        if strdata.startswith("cp:") or strdata.startswith("/bin/cp:"):
+            self._cachedata.append(strdata.partition(": ")[2])
         if "->" in strdata:
             self.value += 1
             if self.maxfiles:
@@ -314,6 +315,14 @@ class Distributive(object):
             raise DistributiveError(_("Failed to create the directory") +
                                     " '%s':\n%s" % (pathname, str(e)))
 
+    def get_squash_size(self):
+        """
+        Получить размер squash образа
+        :return:
+        """
+        raise DistributiveError(_("Squash size unsupported for %s") %
+                                str(self.__class__.__name__))
+
     def _removeDirectory(self, directory):
         """Remove directory and files contained in it"""
         try:
@@ -344,20 +353,21 @@ class Distributive(object):
                                     % (indir, outdir, str(e)))
 
     def rsync(self, fromdir, todir, callbackProgress=None,
-              byfile=None, filesnum=0, **kwargs):
+              byfile=None, filesnum=0, noperm=False, **kwargs):
         """Copy files from 'fromdir' directory to 'todir' directory"""
         cpCmd = getProgPath('/bin/cp')
         if not cpCmd:
             raise DistributiveError(_("'%s' not found") % "cp")
         try:
             joinFrom = partial(path.join, fromdir)
-            params = [cpCmd, "-a", "-x"] + \
+            params = [cpCmd, "-x"] + \
+                     (["--no-preserve=mode,ownership"] if noperm else ["-a"]) + \
                      map(joinFrom,
                          ifilterfalse(byfile or operator.not_,
                                       listDirectory(fromdir))) + \
                      [todir]
             cpProcess = cpProcessProgress(*params,
-                                          maxfiles=filesnum)
+                                          maxfiles=filesnum, stderr=STDOUT)
             for perc in cpProcess.progress():
                 if callbackProgress and not byfile: callbackProgress(perc)
             res = cpProcess.success()
@@ -535,7 +545,8 @@ class Distributive(object):
                 extname = mapExtName.get(distr.__class__, "")
                 image = distr.convertToDirectory()
             except Exception as e:
-                print str(e)
+                # TODO: отладка почему образ не подходит
+                #print str(e)
                 if distr:
                     distr.close()
                 return {}.copy()
@@ -616,6 +627,7 @@ class DirectoryDistributive(Distributive):
 
     def __init__(self, directory, parent=None, mdirectory=None):
         Distributive.__init__(self, parent=parent)
+        self.no_unmount = False
         self.directory = directory
         self.mdirectory = mdirectory
         self.system_mounted = False
@@ -1487,6 +1499,14 @@ class IsoDistributive(Distributive):
         return path.exists(path.join(pathname, "syslinux")) and \
                path.exists(path.join(pathname, "isolinux"))
 
+    def get_squash_size(self):
+        try:
+            dn = self.getIsoContentDirectory()
+            fn = path.join(dn, "livecd.squashfs")
+            return path.getsize(fn)
+        except Exception:
+            raise DistributiveError(_("Failed to get size of the squash image"))
+
     def _mountIso(self, file, directory):
         if self.file != self.mdirectory:
             self._makeDirectory(directory)
@@ -1670,8 +1690,27 @@ class FlashDistributive(PartitionDistributive):
                 self._umountDirectory(mp)
             else:
                 raise DistributiveError(
-                    _("Failed to format %s: this partition is mounted")
-                    % dev)
+                    _("Failed to format %s: this partition is mounted") % dev)
+
+    def performFormat(self):
+        """Perform format for all partition of this distributive"""
+        if not self.isFormat:
+            dn = self.getDirectory()
+            clear_match = re.compile(
+                r"^(boot|efi|isolinux|syslinux|id.*\.uefi|"
+                "ldlinux.c32|ldlinux.sys|livecd|livecd.squashfs)$")
+            for fn in filter(clear_match.match, listDirectory(dn)):
+                full_path = path.join(dn, fn)
+                try:
+                    if path.isdir(full_path):
+                        removeDir(full_path)
+                    else:
+                        os.unlink(full_path)
+                except (OSError, IOError):
+                    raise DistributiveError(
+                        _("Failed to remove %s") % fn)
+        else:
+            super(FlashDistributive, self).performFormat()
 
     def getType(self):
         return _("USB flash %s") % self.partition
@@ -1685,6 +1724,36 @@ class FlashDistributive(PartitionDistributive):
         return path.exists(path.join(pathname, "syslinux")) and \
                path.exists(path.join(pathname, "isolinux"))
 
+    def writeable(self, mp):
+        """
+        Перемонитровать в rw, при необходимости
+        :param dn:
+        :return:
+        """
+        if not check_rw(mp):
+            # перемонитровать в rw
+            p = process('/bin/mount', '-o', 'remount,rw', mp, stderr=STDOUT)
+            if p.success() and check_rw(mp):
+                return True
+            return False
+        else:
+            return True
+
+    def convertToDirectory(self):
+        mp = isMount(self.partition)
+        if mp and self.writeable(mp):
+            d = DirectoryDistributive(mp)
+            d.no_unmount = True
+            return d
+        return super(FlashDistributive, self).convertToDirectory()
+
+    def releaseChild(self, child):
+        """Umount child Directory distributive"""
+        if isinstance(child, DirectoryDistributive):
+            if not child.no_unmount:
+                self._umountPartition(child.directory)
+            child.directory = None
+
     def installFrom(self, source, **kwargs):
         """Install distributive to partition from source distributive"""
         # make temporary directory for creating iso image
@@ -1693,7 +1762,7 @@ class FlashDistributive(PartitionDistributive):
         if isinstance(source, IsoDistributive):
             self.rsync(source.getIsoContentDirectory(), distrTo.directory,
                        byfile=lambda x: x.startswith('livecd.'),
-                       **kwargs)
+                       noperm=True, **kwargs)
         else:
             raise DistributiveError(
                 _("Flash install does not support %s") %
